
local anim = {}
anim.__index = anim

local function bisect(tab, min, max, compare, lerp)
	assert(min <= max,'bad arguments to bisect, min > max')
	local d = (max - min) * 0.5;
	if d == 0 then 
		return tab[min]
	elseif d < 1 then
		return lerp(tab[min], tab[max], compare(tab[min]))
	else
		local m = min + math.floor(d)
		local v = compare(tab[m]) 
		if v == 0 then
			return tab[m]
		elseif v > 0 then
			return bisect(tab, m, max, compare, lerp)
		else
			return bisect(tab, min, m, compare, lerp)
		end
	end
end

local function lerp1(x1, x2, a)
	return a*x1 + (1-a)*x2
end
local function lerpFrame(f1, f2, a)
	return {t = lerp1(f1.t,f2.t,a), x = lerp1(f1.x,f2.x,a), y = lerp1(f1.y,f2.y,a), z = lerp1(f1.z,f2.z,a)}
end

function anim.fromFile(f)
	local data = require(f)
	local tab = {}
	for i = 1,24 do
		tab[i] = {}
	end
	local startTime = data[1].time
	for _,k in ipairs(data) do
		table.insert(tab[k.id+1], {
			t = k.time - startTime, 
			x = k.x, 
			y = k.y, 
			z = k.z
		})
	end
	for i = 1,24 do
		tab[i] = setmetatable(tab[i],{
			__index = function (t,k)
				return bisect(t,1,#t,function(frame) return k - frame.t end, lerpFrame)
			end
		})
	end
	
	return setmetatable({samples=tab, currentTime = 0},anim)
end
function anim:scale(sx,sy,sz)
	sy = sy or sx
	sz = sz or sx
	for _,k in ipairs(self.samples) do
		for _,f in ipairs(k) do
			f.x = f.x * sx
			f.y = f.y * sy
			f.z = f.z * sz
		end
	end
end
function anim:update(dt)
	self.currentTime = (self.currentTime + dt) % self:length()
end
function anim:length()
	local frames = self.samples[1]
	return frames[#frames].t
end
function anim:getJointXY(id)
	local joint = self.samples[id][self.currentTime]
	return joint.x, joint.y
end
function anim:translate(dx,dy,dz)
	dy = dy or dx
	dz = dz or dx
	for _,k in ipairs(self.samples) do
		for _,f in ipairs(k) do
			f.x = f.x + dx
			f.y = f.y + dy
			f.z = f.z + dz
		end
	end
end
return anim
--0	Head		12	Right Elbow
--1	Neck		13	Right Wrist
--2	Torso		14	Right Hand
--3	Waist		15	Right Fingertip
--4	Left Collar		16	Left Hip
--5	Left Shoulder		17	Left Knee
--6	Left Elbow		18	Left Ankle
--7	Left Wrist		19	Left Foot
--8	Left Hand		20	Right Hip
--9	Left Fingertip		21	Right Knee
--10	Right Collar		22	Right Ankle
--11	Right Shoulder		23	Right Foot

