
local csplines = {}
csplines.__index = csplines

local function CRSplineEval(x0, y0, x1, y1, x2, y2, x3, y3, t)
	t = math.max(0, math.min(1, t))
	local t2 = t * t
	local t3 = t * t2
	local retX = 0.5 *((2 * x1) + (-x0 + x2) * t + (2 * x0 - 5*x1 + 4*x2 - x3) * t2 + (-x0 + 3*x1- 3*x2 + x3) * t3)
	local retY = 0.5 *((2 * y1) + (-y0 + y2) * t + (2 * y0 - 5*y1 + 4*y2 - y3) * t2 + (-y0 + 3*y1- 3*y2 + y3) * t3)
	return retX, retY
end

local function computeBoundaryPoints( xo, yo, xe, ye, dxo, dyo, dxe, dye )
	return xo - 2 * dxo, yo - 2 * dyo, xe + 2 * dxe, ye + 2 * dye
end

function csplines.new(xo, yo, xe, ye, dxo, dyo, dxe, dye)
	local t = {}
	local xi, yi, xf, yf = computeBoundaryPoints(xo, yo, xe, ye, dxo or 0, dyo or 0, dxe or 0, dye or 0)
	t.points = {xi, yi, xo, yo, xe, ye, xf, yf}
	t.nbSegments = 1
 	return setmetatable(t,csplines)
end
function csplines.newFromPoints(points)
	return setmetatable({["points"]=points, nbSegments=#points * 0.5 - 3},csplines)
end

function csplines:eval(t)
	local p = self.points
	local time = t * self.nbSegments
	local i = math.floor(time)
	time = time - i
	i = i * 2 + 1
	return CRSplineEval(p[i],p[i+1],p[i+2],p[i+3],p[i+4],p[i+5],p[i+6],p[i+7],time)
end
function csplines:setSplit(n)
	assert(n > 0, "invalid split parameter, should be > 0")
	local step = 1 / n
	local p = self.points
	local newPoints = {p[1], p[2], p[3], p[4]}
	for t = step, 1-step, step do
		local x,y = self:eval(t)
		table.insert(newPoints, x) 
		table.insert(newPoints, y)
	end
	table.insert(newPoints, p[#p - 3])
	table.insert(newPoints, p[#p - 2])
	table.insert(newPoints, p[#p - 1])
	table.insert(newPoints, p[#p])
	self.points = newPoints
	self.nbSegments = n
end
function csplines:getOrigin()
	return self.points[3], self.points[4]
end
function csplines:getExtremity()
	local p = self.points
	return p[#p-3], p[#p-2]
end
function csplines:getLength()
	local p = self.points
	local x = p[#p-3] - p[3]
	local y = p[#p-2] - p[4]
	return math.sqrt(x*x + y*y)
end
function csplines:setOrigin(x, y)
	self.points[1] = x
	self.points[2] = y
	self.points[3] = x
	self.points[4] = y
	return self
end
function csplines:setExtremity(x, y)
	local n =  #self.points
	self.points[n - 3] = x
	self.points[n - 2] = y
	self.points[n - 1] = x
	self.points[n] = y
	return self
end
function csplines:translate(dx, dy)
	local p = self.points
	for i = 1,#p,2 do
		p[i] = p[i] + dx
		p[i+1] = p[i+1] + dy
	end
	return self
end

local function drawSpline(x0, y0, x1, y1, x2, y2, x3, y3, nsteps)
	local xtmp, ytmp = x1, y1
	local step = 1 / nsteps
	local line = love.graphics.line
	for t = step,1-step,step do
		local xnext, ynext = CRSplineEval(x0, y0, x1, y1, x2, y2, x3, y3, t)
		line(xtmp, ytmp, xnext, ynext)
		xtmp, ytmp = xnext, ynext
	end
	line(xtmp, ytmp, x2, y2)
end
function csplines:draw(nsteps)
	local points = self.points
	nsteps = math.max(nsteps or 10, self.nbSegments)
	for i = 1, #points - 7, 2 do
		drawSpline(points[i],points[i+1],points[i+2],points[i+3], points[i+4], points[i+5], points[i+6], points[i+7], nsteps)
	end
end
return csplines
