local width, height
--local points
local time
--local pointsInit
local csplines = require 'csplines'
local animation = require 'anim'

local function gaussian(avg, var, avg2, var2)
	local w, x1, x2
	local ranf = math.random
	repeat 
		x1 = 2.0 * ranf() - 1.0
		x2 = 2.0 * ranf() - 1.0
		w = x1 * x1 + x2 * x2
	until w <= 1.0 

	w = math.sqrt( (-2.0 * math.log( w ) ) / w );
	return (avg or 0) + x1 * w * (var or 1), (avg2 or avg or 0) + x2 * w * (var2 or var or 1)
end

local function shakeInPlace(points, varTan, varNorm)
	for i = 5,#points - 5,2 do
		local dt, dn = gaussian(0,varTan,0,varNorm)
		local tanX = 0.5 * (points[i + 2] - points[i - 2])
		local tanY = 0.5 * (points[i + 3] - points[i - 1])
		local tanN = 1/math.sqrt(tanX * tanX + tanY * tanY)
		tanX, tanY = tanX * tanN, tanY * tanN
		local normX, normY = -tanY, tanX
		points[i] = points[i] + tanX * dt + normX * dn
		points[i+1] = points[i+1] + tanY * dt + normY * dn
	end
	return points
end

local function shake(points, varx, vary)
	local copy = {}
	for i,v in ipairs(points) do
		table.insert(copy,v) -- + math.random(-radius, radius))
	end
	return shakeInPlace(copy, varx, vary)
end

local spline
local splineDraw
local fb
local function handdrawLine(x1,y1,x2,y2,...)
	if not y2 then return end
	local s = csplines.new(x1,y1,x2,y2)
	local length = s:getLength()
	s:setSplit(2)
	shakeInPlace(s.points, 0.2 * length, 0.03 * length)
	s:draw(length / 30)
	handdrawLine(x2, y2, ...)
end
local drawLine = handdrawLine
local function handdrawCircle(style, x, y, r, n)
	n = n or 7
	local step = 2 * math.pi / n
	local arcI = math.pi * 0.5 - step
	local arcF = arcI + 2 * math.pi + 2*step
	local points = {}
	for arc = arcI, arcF, step do
		table.insert(points, x + r * math.cos(arc))
		table.insert(points, y + r * math.sin(arc))
	end
	shakeInPlace(points, 0.1 * r, 0.03 * r)
	csplines.newFromPoints(points):draw(10)
end
local drawCircle = handdrawCircle
function loadBackrounds()
	local lg = love.graphics
	local b = {}
	local scale = 3
	b.scale = scale
	b.img = {}
	for i = 1,9 do
		local img = lg.newImage(string.format('resources/paper512_%d.png',i))
		img:setWrap('repeat','repeat')
		img:setFilter('linear','linear')
		table.insert(b.img,img)
	end
	b.curImg = 1
	b.quad = lg.newQuad(0, 0, scale * width, scale * height, b.img[1]:getWidth(), b.img[1]:getHeight())
	
	local background = {}
	background.__index = background
	function background:draw(dx, dy)
		local lg = love.graphics
		local s = self.scale
		self.quad:setViewport(50+dx, 80+dy, s*width, s*height)
		lg.setColor(255,255,255,255)
		lg.drawq(self.img[self.curImg],self.quad,0,0,0,1/self.scale)
		self.curImg = ((self.curImg + 1) % 9) + 1
	end

	return setmetatable(b,background)
end
local animSkel
local background
function love.load()
	local lg = love.graphics
	width = lg.getWidth()
	height = lg.getHeight()
	fb = lg.isSupported and lg.isSupported('canvas') and lg.newCanvas(width,height) or lg.newFramebuffer(width,height)
	background = loadBackrounds()

	lg.setLineWidth(3)
	lg.setLineStyle('smooth')
	animSkel = animation.fromFile('resources/tracker0.lua')
--	animSkel = animation.fromFile('test.lua')
	animSkel:scale(200,-200)
	animSkel:translate(width/2, height/2,0)
	time = 0
end
local fadeEffect = false
local backgroundEffect = true
local handdrawnEffect = true
local frameTime = 1/10
local fadeStart = frameTime * 2 / 5
local fadeRate = .85
local fadeColor = {80,80,80,255}
local fade = 255

local function drawAnimLine(anim, index1, index2, ...)
	--print("drawAnimLine", anim, time, index1, index2)
	--table.foreach(anim.samples[index1][0.5],print)
	--local samples = anim.samples
	--local joint1 = samples[index1][time]
	--table.foreach(samples,print)
	local x1, y1 = anim:getJointXY(index1)
	local rest = {index2, ...}
	for i = 1, #rest do
		local x2, y2 = anim:getJointXY(rest[i])
		drawLine(x1, y1, x2, y2)
		x1, y1 = x2, y2
	end
end

local function paperDraw()
	love.graphics.setColor(50,50,50,180)
--	drawAnimLine(animSkel, 1, 2) -- head
	do
		local x1,y1 = animSkel:getJointXY(1) -- head
		local x2,y2 = animSkel:getJointXY(2) -- neck
		local l = math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
		drawCircle('line',(3*x1 + 2*x2)*0.2, (3*y1 + 2*y2)*0.2, l * 0.4)
		drawLine((x1 + 4* x2)*0.2, (y1 + 4* y2)*0.2, x2, y2)
	end
	drawAnimLine(animSkel, 2, 6, 7, 10) -- left arm
	drawAnimLine(animSkel, 2, 12, 13, 16) -- right arm
	local x1,y1 = animSkel:getJointXY(17) -- left hip
	local x2,y2 = animSkel:getJointXY(21) -- right hip
	x2,y2 = (x1 + x2) / 2, (y1 + y2) / 2 -- between hips
	x1,y1 = animSkel:getJointXY(2) -- neck end
	drawLine(x1,y1,x2,y2) -- trunk
	x1,y1 = x2, y2 -- between hips
	x2,y2 = animSkel:getJointXY(18) -- left knee
	drawLine(x1,y1,x2,y2) -- left leg
	x2,y2 = animSkel:getJointXY(22) -- right knee
	drawLine(x1,y1,x2,y2) -- right leg
	drawAnimLine(animSkel, 18, 20) -- finish legs
	drawAnimLine(animSkel, 22, 24)
end
function love.update(dt)
	time = time + dt
	animSkel:update(dt)
	if fadeEffect and time >= fadeStart then
		fade = fade * fadeRate
	end
	if time >= frameTime then
		local lg = love.graphics
		fb:renderTo(function()
			local dx,dy
			if backgroundEffect then
				dx, dy = gaussian(0, 2)
			else
				dx, dy = 0,0
			end
			background:draw(dx,dy)
			lg.push()
			lg.translate(-dx,-dy)
			paperDraw()
			lg.pop()
			time = 0
			fade = 255
		end)
		local x = gaussian(0,1/30)
		frameTime = math.min(0.15 + x, 1/15)
--		frameTime = 1/15
		fadeStart = frameTime * 2 / 5
	end
end

function love.draw()
	local lg = love.graphics
	lg.setColor(255,255,255,255)
	lg.draw(fb,0,0)
	if fadeEffect then
		fadeColor[4] = 1 - fade
		lg.setColor(unpack(fadeColor))
		lg.rectangle('fill',0,0,width,height)
	end
end

function love.keypressed(k, u)
	if k == 'b' then
		backgroundEffect = not backgroundEffect
	elseif k == 'f' then
		fadeEffect = not fadeEffect
	elseif k == 'l' then
		handdrawnEffect = not handdrawnEffect
		drawLine = handdrawnEffect and handdrawLine or love.graphics.line
		drawCircle = handdrawnEffect and handdrawCircle or love.graphics.circle
	end
end
